package com.landray.kmss.sys.attachment.io;

import java.io.IOException;
import java.io.InputStream;
import java.io.RandomAccessFile;

public class RandomAccessFileInputStream extends AbstractInputStream
{
    private RandomAccessFile file;
    private long currentPosition;
    private long endPosition;
    private boolean noEncryption;
    private boolean isAESEncryption;
    private boolean isSM4Encryption;
    
    public static byte[] markBytes() {
        return RandomAccessFileInputStream.markBytes;
    }
    
    public RandomAccessFileInputStream(final RandomAccessFile raFile, final long startPosition, final long length) throws Exception {
        super(null);
        this.file = raFile;
        this.currentPosition = startPosition;
        this._init();
        this.endPosition = this.currentPosition + length;
    }
    
    @Override
    public int available() {
        return (int)(this.endPosition - this.currentPosition);
    }
    
    @Override
    public void close() throws IOException {
        this.file.close();
    }
    
    @Override
    public int read() throws IOException {
        synchronized (this.file) {
            int retval = -1;
            if (this.currentPosition < this.endPosition) {
                this.file.seek(this.currentPosition);
                ++this.currentPosition;
                retval = this.file.read();
            }
            final int n = retval;
            monitorexit(this.file);
            return n;
        }
    }
    
    @Override
    public int read(final byte[] b) throws IOException {
        return this.read(b, 0, b.length);
    }
    
    @Override
    public int read(final byte[] b, final int offset, int length) throws IOException {
        if (length > this.available()) {
            length = this.available();
        }
        int amountRead = -1;
        if (this.available() > 0) {
            synchronized (this.file) {
                this.file.seek(this.currentPosition);
                amountRead = this.file.read(b, offset, length);
                this.format(b, offset, amountRead);
                monitorexit(this.file);
            }
        }
        if (amountRead > 0) {
            this.currentPosition += amountRead;
        }
        return amountRead;
    }
    
    @Override
    public long skip(final long amountToSkip) {
        final long amountSkipped = Math.min(amountToSkip, this.available());
        this.currentPosition += amountSkipped;
        return amountSkipped;
    }
    
    private void _init() throws Exception {
        this.noEncryption = (this.cryptionMode == 0);
        if (this.noEncryption) {
            return;
        }
        if (this.cryptionMode != 0) {
            this.initAES();
            this.initSM4();
        }
        final int len = RandomAccessFileInputStream.markBytes.length;
        final byte[] b = new byte[len];
        this.file.seek(0L);
        this.file.read(b);
        this.isAESEncryption = this.isAESEncryption(b);
        if (this.isAESEncryption) {
            this.currentPosition += len;
            return;
        }
        this.isSM4Encryption = this.isSM4Encryption(b);
        if (this.isSM4Encryption) {
            this.currentPosition += len;
            return;
        }
        for (int i = 0; i < len; ++i) {
            if (this.format(b[i] + 128) != RandomAccessFileInputStream.markBytes[i] + 128) {
                this.noEncryption = true;
                return;
            }
        }
        this.currentPosition += len;
    }
    
    @Override
    protected void init() throws IOException {
    }
    
    @Override
    protected void format(final byte[] b, final int off, final int len) {
        if (this.noEncryption) {
            return;
        }
        for (int i = off; i < len; ++i) {
            if (this.isAESEncryption) {
                b[i] = this.aesCrypt.decrypt(b, i, 1)[0];
            }
            else if (this.isSM4Encryption) {
                b[i] = this.sm4Crypt.decrypt(b, i, 1)[0];
            }
            else if (b[i] == -128) {
                b[i] = 127;
            }
            else {
                final int tmp35_33 = i;
                final byte[] tmp35_34 = b;
                --tmp35_34[tmp35_33];
            }
        }
    }
    
    @Override
    protected int format(final int i) {
        if (this.noEncryption) {
            return i;
        }
        if (i == -1) {
            return -1;
        }
        if (this.isAESEncryption) {
            final byte[] b = this.aesCrypt.decrypt(new byte[] { (byte)i });
            return b[0];
        }
        if (this.isSM4Encryption) {
            final byte[] b = this.sm4Crypt.decrypt(new byte[] { (byte)i });
            return b[0];
        }
        if (i == 0) {
            return 255;
        }
        return i - 1;
    }
}
